#include <ansi_c.h>
#include <visa.h>
#include <cvirte.h>     
#include <userint.h>
#include "rssma.h"
#include "RSSMA_List_Mode_Example.h"


	/* Values for control Active Output */
#define RSSMA_LIST_MODE_AUTO			0
#define RSSMA_LIST_MODE_SINGLE			1
#define RSSMA_LIST_MODE_STEP			2
#define RSSMA_LIST_MODE_EXT_SINGLE		3
#define RSSMA_LIST_MODE_EXT_STEP		4
#define RSSMA_LIST_MODE_EXT_HOP			5
#define RSSMA_LIST_MODE_EXT_HOP_DIRECT	6


/* Error check macro for driver functions */
#define CHECKERR( fCal ) \
    { \
        error = (fCal);\
        rssma_error_message (io, error, error_message);\
        if (error < VI_SUCCESS) {\
            sprintf (error_buffer, "Primary Error: 0x%08X, %s\n", error, error_message);\
            MessagePopup ("Error", error_buffer); \
            rssma_error_query (io, &error, error_message);\
            sprintf (error_buffer, "Secondary Error: 0x%08X, %s\n", error, error_message);\
            SetCtrlAttribute (panelHandle, PANEL_APPLY, ATTR_DIMMED, 0);\
            SetCtrlAttribute (panelHandle, PANEL_Exit, ATTR_DIMMED, 0);\
            SetWaitCursor (0); \
            MessagePopup ("Error", error_buffer); \
            rssma_close(io); \
            io = NULL;\
            return -1; \
        } \
    }
    
/* Globals */
static int panelHandle;

ViStatus  error;
ViSession io = NULL;
ViChar    error_message[256];
ViChar    error_buffer[256];

/* Local Functions */
int initialize (void);
void ShapeChange (void);

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)
        return -1;  /* out of memory */
    if ((panelHandle = LoadPanel (0, "RSSMA_List_Mode_Example.uir", PANEL)) < 0)
        return -1;
    
    DisplayPanel (panelHandle);
    RunUserInterface ();
    DiscardPanel (panelHandle);
    
    return 0;
}

/*===========================================================================*/
/* Function: Apply Settings                                                  */
/* Purpose:  This is a callback function of Apply button. The values are     */
/*           send to the instrument                                          */
/*===========================================================================*/
int CVICALLBACK apllySettings (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
	ViBoolean 	rfState;	
	ViInt32		listMode,
				listIndex,
				trigSlope;
	ViReal64	dwellTime;
	ViChar		dirName[256],
				list[256],
				listName[512];
	
    switch (event)
    {
        case EVENT_COMMIT:
            /* Initialize driver */

			if (io == VI_NULL)
            {
                if (initialize() < VI_SUCCESS) break;
            }

            SetCtrlAttribute (panelHandle, PANEL_APPLY, ATTR_DIMMED, 1);
            SetCtrlAttribute (panelHandle, PANEL_Exit, ATTR_DIMMED, 1);
            ProcessDrawEvents ();

            SetWaitCursor(1);

            /* Get values from controls */
            GetCtrlVal (panelHandle, PANEL_RF_STATE, &rfState);
			GetCtrlIndex (panelHandle, PANEL_LIST_MODE, &listMode);			
            GetCtrlVal (panelHandle, PANEL_DWELL_TIME, &dwellTime);
            GetCtrlVal (panelHandle, PANEL_DIR_NAME, dirName);
            GetCtrlIndex (panelHandle, PANEL_LIST, &listIndex);						

            /* Turn On/Off RF output at the end of sequence */
            CHECKERR (rssma_SetOutputState(io, rfState));
			
			if(rfState)
			{
				if (listIndex != -1)
				{					
					GetLabelFromIndex(panelHandle, PANEL_LIST, listIndex, list);
				
					sprintf(listName, "%s\\%s", dirName, list);
				
					CHECKERR( rssma_SetLISTSelectList(io, listName));
			
					CHECKERR( rssma_SetLISTMode (io, listMode));
					CHECKERR( rssma_SetLISTDwellTime(io, dwellTime));
					
					switch(listMode)
					{
						case RSSMA_LIST_MODE_EXT_SINGLE:
						case RSSMA_LIST_MODE_EXT_STEP:
						case RSSMA_LIST_MODE_EXT_HOP:				
							{
            					GetCtrlIndex (panelHandle, PANEL_TRIG_SLOPE, &trigSlope);
								
								CHECKERR( rssma_SetInstrumentTriggerInputSlope(io, trigSlope));
							}
							break;
					}
				}				
			}

            SetCtrlAttribute (panelHandle, PANEL_APPLY, ATTR_DIMMED, 0);
            SetCtrlAttribute (panelHandle, PANEL_Exit, ATTR_DIMMED, 0);

            SetWaitCursor(0);

            break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: Fill List                                                       */
/* Purpose:  This is a callback function of Fill List button. It fills the   */
/*           List ring.														 */
/*===========================================================================*/
int CVICALLBACK fillList (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
	ViChar	dirName[256],
			catalogList[256],
			item[256],
			*p2item;
	ViInt32	itemValue = 0;
	
	
	switch (event)
    {
        case EVENT_COMMIT:
			if (io == VI_NULL)
	        {
	        	if (initialize() < VI_SUCCESS) break;
	        }			
		
			/* delete all ring items.*/
			DeleteListItem(panelHandle, PANEL_LIST, 0, -1);
			
			GetCtrlVal(panelHandle, PANEL_DIR_NAME, dirName);

			if(dirName)
			{
				CHECKERR (rssma_MMEMoryChangeDirectory(io, dirName));
			}
			CHECKERR (rssma_LISTCatalog(io, VI_NULL, 256, catalogList));
			
			p2item = strtok(catalogList, ",");
			while (p2item)
			{
				/* insert new item to ring control. */
				InsertListItem(panelHandle, PANEL_LIST, -1, p2item, itemValue++);					
			   	p2item = strtok(NULL, ",");
			}	
			break;
    }
    return 0;
}

/*===========================================================================*/
/* Function: listModeRing                                                    */
/* Purpose:  This is a callback function of List Mode Ring.                  */
/*===========================================================================*/
int CVICALLBACK listModeRing (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
	ViInt32		mode;
	ViBoolean	bDimmed = VI_TRUE;

    switch (event)
    {
        case EVENT_COMMIT:
		{
			GetCtrlIndex (panelHandle, PANEL_LIST_MODE, &mode);				
	
			switch(mode)
			{
				case RSSMA_LIST_MODE_EXT_SINGLE:
				case RSSMA_LIST_MODE_EXT_STEP:
				case RSSMA_LIST_MODE_EXT_HOP:				
					bDimmed = VI_FALSE;
					break;
			}
		}						
			SetCtrlAttribute (panelHandle, PANEL_TRIG_SLOPE, ATTR_DIMMED, bDimmed);								
    }
    return 0;
}

/*===========================================================================*/
/* Function: Panel Help                                                      */
/* Purpose:  This is a callback function of the menu bar. It displays a help */
/*           window describing panel being used.                             */
/*===========================================================================*/
void CVICALLBACK panelHelp (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    if (panel == panelHandle)
        MessagePopup ("Help", "This is a simple demo program using\n"
                              "the Rohde&Schwarz SMA Signal Generator\n"
                              "instrument driver provided in the package.\n");
}

/*===========================================================================*/
/* Function: Exit                                                            */
/* Purpose:  This is a callback function of Exit button. It closes the       */
/*           session and exits the example.                                  */
/*===========================================================================*/
int CVICALLBACK exitApp (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
    {
        case EVENT_COMMIT:
            if (io) rssma_close (io);
            QuitUserInterface (0);
            break;
    }
    return 0;
}

/*===========================================================================*/ 
/* Local Functions Definitions                                               */
/*===========================================================================*/

/*===========================================================================*/
/* Function: Initialize                                                      */
/* Purpose:  This function initializes the instrument, performs ID Query     */
/*           and Reset                                                       */
/*===========================================================================*/
int initialize (void)
{
    ViChar      instr_desc[256];
    ViBoolean   id;
    ViBoolean   rst;


    SetWaitCursor(0);
    GetCtrlVal (panelHandle, PANEL_INSTR_DESC, instr_desc);
    GetCtrlVal (panelHandle, PANEL_IDQUERY, &id);
    GetCtrlVal (panelHandle, PANEL_RESET, &rst);
    SetWaitCursor(1);
    if ((error = rssma_init (instr_desc, id, rst, &io)) < 0 )
    {
        rssma_error_message(VI_NULL, error, error_message);
        sprintf (error_buffer, "Initialization Error:\n\n%s\n\nCheck your "
                "connections and make sure you have the right instrument descriptor.",
                 error_message);
        MessagePopup ("ERROR!", error_buffer);
        SetWaitCursor(0);
        return(-1);   
    }

    viSetAttribute (io, VI_ATTR_TMO_VALUE, 10000);

    SetWaitCursor(0);
    return 0;
}

